﻿// maindlg.cpp : implementation of the CMainDlg class
//
/////////////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "resource.h"

#include "aboutdlg.h"
#include "maindlg.h"
#include "StationSpyPlugin.h"

CMainDlg::CMainDlg(StationSpyPlugin *plugin) : 
	CDialogImpl<CMainDlg>(), CUpdateUI<CMainDlg>(), CMessageFilter(), CIdleHandler()
{
	m_pPlugin = plugin;
}

BOOL CMainDlg::PreTranslateMessage(MSG* pMsg)
{
	return CWindow::IsDialogMessage(pMsg);
}

BOOL CMainDlg::OnIdle()
{
	return FALSE;
}

LRESULT CMainDlg::OnInitDialog(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/)
{
	// Move the dialog at the center on the screen
	CenterWindow();

	// Set icons
	HICON hIcon = (HICON)::LoadImage(_Module.GetResourceInstance(), MAKEINTRESOURCE(IDR_MAINFRAME), 
		IMAGE_ICON, ::GetSystemMetrics(SM_CXICON), ::GetSystemMetrics(SM_CYICON), LR_DEFAULTCOLOR);
	SetIcon(hIcon, TRUE);
	HICON hIconSmall = (HICON)::LoadImage(_Module.GetResourceInstance(), MAKEINTRESOURCE(IDR_MAINFRAME), 
		IMAGE_ICON, ::GetSystemMetrics(SM_CXSMICON), ::GetSystemMetrics(SM_CYSMICON), LR_DEFAULTCOLOR);
	SetIcon(hIconSmall, FALSE);

	// Register an object for a message filtering and idle updates
	CMessageLoop* pLoop = _Module.GetMessageLoop();
	ATLASSERT(pLoop != NULL);
	pLoop->AddMessageFilter(this);
	pLoop->AddIdleHandler(this);

	UIAddChildWindowContainer(m_hWnd);

	m_MessageList.Attach(GetDlgItem(IDC_LIST1));
	m_CheckOnDraw.Attach(GetDlgItem(IDC_DRAW));
	m_CheckOnUpdateScene.Attach(GetDlgItem(IDC_UPDATE_SCENE));
	m_CheckOnUpdateUndo.Attach(GetDlgItem(IDC_UPDATE_UNDO));

	return TRUE;
}

LRESULT CMainDlg::OnAppAbout(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
{
	CAboutDlg dlg;
	dlg.DoModal();
	return 0;
}

LRESULT CMainDlg::OnOK(WORD /*wNotifyCode*/, WORD wID, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
{
	// TODO: Add validation code 
	CloseDialog(wID);
	return 0;
}

LRESULT CMainDlg::OnCancel(WORD /*wNotifyCode*/, WORD wID, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
{
	CloseDialog(wID);
	return 0;
}

LRESULT CMainDlg::OnClear(WORD /*wNotifyCode*/, WORD wID, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
{
	// Clear contents in a message list
	// メッセージリストの内容を消去
	m_MessageList.ResetContent();
	return 0;
}

LRESULT CMainDlg::OnCheckOnDraw(WORD /*wNotifyCode*/, WORD wID, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
{
	m_pPlugin->m_bOnDraw = m_CheckOnDraw.GetCheck() ? true : false;
	return 0;
}

LRESULT CMainDlg::OnCheckOnUpdateScene(WORD /*wNotifyCode*/, WORD wID, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
{
	m_pPlugin->m_bOnUpdateScene = m_CheckOnUpdateScene.GetCheck() ? true : false;
	return 0;
}

LRESULT CMainDlg::OnCheckOnUpdateUndo(WORD /*wNotifyCode*/, WORD wID, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
{
	m_pPlugin->m_bOnUpdateUndo = m_CheckOnUpdateUndo.GetCheck() ? true : false;
	return 0;
}

// Add a message log
// メッセージログの追加
void CMainDlg::AddMessage(const char *message)
{
	m_MessageList.AddString(message);
	m_MessageList.SetTopIndex(m_MessageList.GetCount()-1);
}

void CMainDlg::CloseDialog(int nVal)
{
	// Hide the window
	// ウインドウを隠す
	ShowWindow(SW_HIDE);

	// Notify from the plug-in
	// プラグインから通知
	m_pPlugin->m_bActivate = false;
	m_pPlugin->WindowClose();

//	DestroyWindow();
//	::PostQuitMessage(nVal);
}
